/*
	SDL - Simple DirectMedia Layer
	Copyright (C) 1997-2012 Sam Lantinga

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

	Sam Lantinga
	slouken@libsdl.org
*/

#include <gint/gint.h>
#include <gint/timer.h>
#include <gint/mpu/tmu.h>

#include "SDL_config.h"

#ifdef SDL_TIMER_PRIZM

#include "SDL_timer.h"
#include "../SDL_timer_c.h"


int timerID = -1;
uint32_t timerTick = 0;
uint32_t timerStart = 0;
uint32_t volatile *Tcnt = NULL;

static int callback(void)
{
	return TIMER_CONTINUE;
}


void SDL_StartTicks(void)
{
    timerID = timer_configure( TIMER_ETMU, 0xFFFFFFFF, GINT_CALL(callback));
    if (timerID!=-1)
    {
        timer_start(timerID);
        Tcnt = &SH7305_ETMU[timerID-3].TCNT;
        timerStart = *Tcnt;
    }
}

Uint32 SDL_GetTicks (void)
{

    if (timerID!=-1)
    {
        timerTick = timerStart - *Tcnt;
        return (timerTick >> 5);
    }
    else return 0xFFFFFFFF;
}

void SDL_Delay (Uint32 ms)
{
    if (timerID!=-1)
    {
        uint32_t timerTempStart = SDL_GetTicks();
        uint32_t currentTCNT;

        do
        {
            currentTCNT = SDL_GetTicks() - timerTempStart;
        }
        while(currentTCNT <= ms);
    }
}


#include "SDL_thread.h"

/* Data to handle a single periodic alarm */
static int timer_alive = 0;
static SDL_Thread *timer = NULL;

static int RunTimer(void *unused)
{
	while ( timer_alive ) {
		if ( SDL_timer_running ) {
			SDL_ThreadedTimerCheck();
		}
		SDL_Delay(1);
	}
	return(0);
}

/* This is only called if the event thread is not running */
int SDL_SYS_TimerInit(void)
{
	timer_alive = 1;
	timer = SDL_CreateThread(RunTimer, NULL);
	if ( timer == NULL )
		return(-1);
	return(SDL_SetTimerThreaded(1));
}

void SDL_SYS_TimerQuit(void)
{
	timer_alive = 0;
	if ( timer ) {
		SDL_WaitThread(timer, NULL);
		timer = NULL;
	}
}

int SDL_SYS_StartTimer(void)
{
	//SDL_SetError("Timers not implemented on the TI-Nspire");
	return(-1);
}

void SDL_SYS_StopTimer(void)
{
	return;
}

#endif /* SDL_TIMER_PRIZM */
